onestep.heterogeneous.nll <- function(theta, z.obs, n1, n2, alpha=0.025){
	delta <- theta[1]						# True population average effect size
	tau <- theta[2]							# Heterogeneity
	w <- theta[3]							# Relative likelihood non-stat sig study is reported
	z.cut <- qnorm(1-alpha)
	d.obs <- z.obs / sqrt((n1*n2)/(n1+n2))
	d.cut <- z.cut / sqrt((n1*n2)/(n1+n2))
	k <- sum(z.obs<z.cut)

	s <- sqrt(tau^2 + 1/n1 + 1/n2)
	ll <- ifelse(k==0, 0, k*log(w))			# Equivalent to defining 0*log(0)=0 as is common
	ll <- ll + sum(dnorm(d.obs, delta, s, log=TRUE))
	ll <- ll - sum(log( w*pnorm(d.cut,delta,s) + (1 - pnorm(d.cut,delta,s)) ))
	-ll
}

estimate.z.no.selection.homogeneous <- function(z.obs,n1,n2,interval){
	tmpf <- function(delta,z.obs,n1,n2){ onestep.heterogeneous.nll(c(delta,0,1), z.obs, n1, n2)  }
	tmpo <- optimize(tmpf, interval, z.obs=z.obs, n1=n1, n2=n2)
	tmph <- optimHess(tmpo$minimum, tmpf, z.obs=z.obs, n1=n1, n2=n2)
	return(c(tmpo$minimum, sqrt(1/tmph), -tmpo$objective))	
}

meta.fe <- function(y,v){ 	# Equivalent to the above but analytic and faster
	vv <- 1 / sum(1/v)
	yy <- sum(y/v) * vv
	return(c(yy,sqrt(vv)))
}

estimate.z.extreme.selection.homogeneous <- function(z.obs,n1,n2,alpha,interval){
	p.value <- 1 - pnorm(z.obs)
	sel <- z.obs > 0 & p.value < alpha
	if(sum(sel)==0){ return(c(NA,NA,NA)) }
	
	z.obs <- z.obs[sel]
	n1 <- n1[sel]
	n2 <- n2[sel]

	tmpf <- function(delta,z.obs,n1,n2,alpha){ onestep.heterogeneous.nll(c(delta,0,0), z.obs, n1, n2, alpha) }
	tmpo <- optimize(tmpf, interval, z.obs=z.obs, n1=n1, n2=n2, alpha=alpha)
	tmph <- optimHess(tmpo$minimum, tmpf, z.obs=z.obs, n1=n1, n2=n2, alpha=alpha)
	return(c(tmpo$minimum, sqrt(1/tmph), -tmpo$objective))	
}

average.power <- function(delta, n1, n2, alpha){
	s <- sqrt(1/n1 + 1/n2)
	z.cut <- qnorm(1-alpha, 0, s)
	power <- 1 - pnorm(z.cut, delta, s) 
	return(mean(power))
}

get.n <- function(delta, target.power, alpha){	
	za <- qnorm(1-alpha)
	zb <- qnorm(1-target.power)
	2 * (za-zb)^2 / delta^2
}

